"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformRawValue = exports.getStylesForProperty = exports.getPropertyName = exports["default"] = void 0;
var _camelize = _interopRequireDefault(require("camelize"));
var _postcssValueParser = _interopRequireDefault(require("postcss-value-parser"));
var _TokenStream = _interopRequireDefault(require("./TokenStream"));
var _index = _interopRequireDefault(require("./transforms/index"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); } /* eslint-disable no-param-reassign */
// Note if this is wrong, you'll need to change tokenTypes.js too
var numberOrLengthRe = /^([+-]?(?:\d*\.)?\d+(?:[Ee][+-]?\d+)?)((?:px)|(?:vw$)|(?:vh$)|(?:vmin$)|(?:vmax$))?$/i;
var boolRe = /^true|false$/i;
var nullRe = /^null$/i;
var undefinedRe = /^undefined$/i;

// Undocumented export
var transformRawValue = exports.transformRawValue = function transformRawValue(input) {
  var value = input.trim();
  var numberMatch = value.match(numberOrLengthRe);
  if (numberMatch !== null) {
    var num = Number(numberMatch[1]);
    var unit = numberMatch[2];
    var isViewportUnit = ['vw', 'vh', 'vmin', 'vmax'].includes(unit);
    if (isViewportUnit) {
      return "scaleVu2dp(".concat(num, ", '").concat(unit, "')");
    } else if (/(\d+)px/.test(value)) {
      return "scalePx2dp(".concat(num, ")");
    } else {
      return num;
    }
  }
  var boolMatch = input.match(boolRe);
  if (boolMatch !== null) return boolMatch[0].toLowerCase() === 'true';
  var nullMatch = input.match(nullRe);
  if (nullMatch !== null) return null;
  var undefinedMatch = input.match(undefinedRe);
  if (undefinedMatch !== null) return undefined;
  return value;
};
var baseTransformShorthandValue = function baseTransformShorthandValue(propName, inputValue) {
  // const ast = parse(inputValue.trim().replace(/PX|Px|pX$/g, ""));
  var ast = (0, _postcssValueParser["default"])(inputValue);
  var tokenStream = new _TokenStream["default"](ast.nodes);
  return _index["default"][propName](tokenStream);
};
var checkBaseTransformShorthandValue = function checkBaseTransformShorthandValue(propName, inputValue) {
  try {
    return baseTransformShorthandValue(propName, inputValue);
  } catch (e) {
    throw new Error("".concat(e.message, " Failed to parse declaration \"").concat(propName, ": ").concat(inputValue, "\""));
  }
};
var transformShorthandValue = process.env.NODE_ENV === 'production' ? baseTransformShorthandValue : checkBaseTransformShorthandValue;
var getStylesForProperty = exports.getStylesForProperty = function getStylesForProperty(propName, inputValue, allowShorthand) {
  var isRawValue = allowShorthand === false || !(propName in _index["default"]);
  var propValue = isRawValue ? transformRawValue(inputValue) : transformShorthandValue(propName, inputValue.trim());
  return propValue && propValue.$merge ? propValue.$merge : _defineProperty({}, propName, propValue);
};
var getPropertyName = exports.getPropertyName = function getPropertyName(propName) {
  var isCustomProp = /^--\w+/.test(propName);
  if (isCustomProp) {
    return propName;
  }
  return (0, _camelize["default"])(propName);
};
var _default = exports["default"] = function _default(rules) {
  var shorthandBlacklist = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return rules.reduce(function (accum, rule) {
    var propertyName = getPropertyName(rule[0]);
    var value = rule[1];
    var allowShorthand = shorthandBlacklist.indexOf(propertyName) === -1;
    return Object.assign(accum, getStylesForProperty(propertyName, value, allowShorthand));
  }, {});
};